@@ -9,7 +9,8 @@ import Stats from './utils/Stats.js';
 
 const warned = {
 	insertLeft: false,
-	insertRight: false
+	insertRight: false,
+	storeName: false
 };
 
 export default function MagicString ( string, options = {} ) {
@@ -54,7 +55,7 @@ MagicString.prototype = {
 	appendLeft ( index, content ) {
 		if ( typeof content !== 'string' ) throw new TypeError( 'inserted content must be a string' );
 
-		if ( DEBUG ) this.stats.time( 'insertLeft' );
+		if ( DEBUG ) this.stats.time( 'appendLeft' );
 
 		this._split( index );
 
@@ -66,14 +67,14 @@ MagicString.prototype = {
 			this.intro += content;
 		}
 
-		if ( DEBUG ) this.stats.timeEnd( 'insertLeft' );
+		if ( DEBUG ) this.stats.timeEnd( 'appendLeft' );
 		return this;
 	},
 
 	appendRight ( index, content ) {
 		if ( typeof content !== 'string' ) throw new TypeError( 'inserted content must be a string' );
 
-		if ( DEBUG ) this.stats.time( 'insertLeft' );
+		if ( DEBUG ) this.stats.time( 'appendRight' );
 
 		this._split( index );
 
@@ -85,7 +86,7 @@ MagicString.prototype = {
 			this.outro += content;
 		}
 
-		if ( DEBUG ) this.stats.timeEnd( 'insertLeft' );
+		if ( DEBUG ) this.stats.timeEnd( 'appendRight' );
 		return this;
 	},
 
@@ -115,9 +116,7 @@ MagicString.prototype = {
 		cloned.lastChunk = clonedChunk;
 
 		if ( this.indentExclusionRanges ) {
-			cloned.indentExclusionRanges = typeof this.indentExclusionRanges[0] === 'number' ?
-				[ this.indentExclusionRanges[0], this.indentExclusionRanges[1] ] :
-				this.indentExclusionRanges.map( range => [ range.start, range.end ] );
+			cloned.indentExclusionRanges = this.indentExclusionRanges.slice();
 		}
 
 		Object.keys( this.sourcemapLocations ).forEach( loc => {
@@ -236,13 +235,9 @@ MagicString.prototype = {
 							if ( charIndex === chunk.start ) {
 								chunk.prependRight( indentStr );
 							} else {
-								const rhs = chunk.split( charIndex );
-								rhs.prependRight( indentStr );
-
-								this.byStart[ charIndex ] = rhs;
-								this.byEnd[ charIndex ] = chunk;
-
-								chunk = rhs;
+								this._splitChunk( chunk, charIndex );
+								chunk = chunk.next;
+								chunk.prependRight( indentStr );
 							}
 						}
 					}
@@ -261,7 +256,7 @@ MagicString.prototype = {
 	},
 
 	insert () {
-		throw new Error( 'magicString.insert(...) is deprecated. Use insertRight(...) or insertLeft(...)' );
+		throw new Error( 'magicString.insert(...) is deprecated. Use prependRight(...) or appendLeft(...)' );
 	},
 
 	insertLeft ( index, content ) {
@@ -314,7 +309,7 @@ MagicString.prototype = {
 		}
 
 		first.previous = newLeft;
-		last.next = newRight;
+		last.next = newRight || null;
 
 		if ( !newLeft ) this.firstChunk = first;
 		if ( !newRight ) this.lastChunk = last;
@@ -323,20 +318,31 @@ MagicString.prototype = {
 		return this;
 	},
 
-	overwrite ( start, end, content, storeName ) {
+	overwrite ( start, end, content, options ) {
 		if ( typeof content !== 'string' ) throw new TypeError( 'replacement content must be a string' );
 
 		while ( start < 0 ) start += this.original.length;
 		while ( end < 0 ) end += this.original.length;
 
 		if ( end > this.original.length ) throw new Error( 'end is out of bounds' );
-		if ( start === end ) throw new Error( 'Cannot overwrite a zero-length range – use insertLeft or insertRight instead' );
+		if ( start === end ) throw new Error( 'Cannot overwrite a zero-length range – use appendLeft or prependRight instead' );
 
 		if ( DEBUG ) this.stats.time( 'overwrite' );
 
 		this._split( start );
 		this._split( end );
 
+		if ( options === true ) {
+			if ( !warned.storeName ) {
+				console.warn( 'The final argument to magicString.overwrite(...) should be an options object. See https://github.com/rich-harris/magic-string' ); // eslint-disable-line no-console
+				warned.storeName = true;
+			}
+
+			options = { storeName: true };
+		}
+		const storeName = options !== undefined ? options.storeName : false;
+		const contentOnly = options !== undefined ? options.contentOnly : false;
+
 		if ( storeName ) {
 			const original = this.original.slice( start, end );
 			this.storedNames[ original ] = true;
@@ -346,17 +352,21 @@ MagicString.prototype = {
 		const last = this.byEnd[ end ];
 
 		if ( first ) {
-			first.edit( content, storeName );
-
-			if ( last ) {
-				first.next = last.next;
-			} else {
-				first.next = null;
-				this.lastChunk = first;
+			if ( end > first.end && first.next !== this.byStart[ first.end ] ) {
+				throw new Error( 'Cannot overwrite across a split point' );
 			}
 
-			first.original = this.original.slice( start, end );
-			first.end = end;
+			first.edit( content, storeName, contentOnly );
+
+			if ( first !== last ) {
+				let chunk = first.next;
+				while ( chunk !== last ) {
+					chunk.edit( '', false );
+					chunk = chunk.next;
+				}
+
+				chunk.edit( '', false );
+			}
 		}
 
 		else {
@@ -426,7 +436,23 @@ MagicString.prototype = {
 		if ( start < 0 || end > this.original.length ) throw new Error( 'Character is out of bounds' );
 		if ( start > end ) throw new Error( 'end must be greater than start' );
 
-		return this.overwrite( start, end, '', false );
+		if ( DEBUG ) this.stats.time( 'remove' );
+
+		this._split( start );
+		this._split( end );
+
+		let chunk = this.byStart[ start ];
+
+		while ( chunk ) {
+			chunk.intro = '';
+			chunk.outro = '';
+			chunk.edit( '' );
+
+			chunk = end > chunk.end ? this.byStart[ chunk.end ] : null;
+		}
+
+		if ( DEBUG ) this.stats.timeEnd( 'remove' );
+		return this;
 	},
 
 	slice ( start = 0, end = this.original.length ) {
@@ -556,10 +582,13 @@ MagicString.prototype = {
 
 			// if chunk was trimmed, we have a new lastChunk
 			if ( chunk.end !== end ) {
-				this.lastChunk = chunk.next;
+				if ( this.lastChunk === chunk ) {
+					this.lastChunk = chunk.next;
+				}
 
 				this.byEnd[ chunk.end ] = chunk;
 				this.byStart[ chunk.next.start ] = chunk.next;
+				this.byEnd[ chunk.next.end ] = chunk.next;
 			}
 
 			if ( aborted ) return this;
@@ -587,6 +616,7 @@ MagicString.prototype = {
 
 				this.byEnd[ chunk.end ] = chunk;
 				this.byStart[ chunk.next.start ] = chunk.next;
+				this.byEnd[ chunk.next.end ] = chunk.next;
 			}
 
 			if ( aborted ) return this;
