@@ -1,6 +1,6 @@
 const assert = require( 'assert' );
 const SourceMapConsumer = require( 'source-map' ).SourceMapConsumer;
-const MagicString = require( '../' );
+const MagicString = require( './utils/IntegrityCheckingMagicString' );
 
 require( 'source-map-support' ).install();
 require( 'console-group' ).install();
@@ -42,12 +42,12 @@ describe( 'MagicString', () => {
 		it( 'preserves intended order', () => {
 			const s = new MagicString( '0123456789' );
 
-			s.insertLeft( 5, 'A' );
-			s.insertRight( 5, 'a' );
-			s.insertRight( 5, 'b' );
-			s.insertLeft( 5, 'B' );
-			s.insertLeft( 5, 'C' );
-			s.insertRight( 5, 'c' );
+			s.appendLeft( 5, 'A' );
+			s.prependRight( 5, 'a' );
+			s.prependRight( 5, 'b' );
+			s.appendLeft( 5, 'B' );
+			s.appendLeft( 5, 'C' );
+			s.prependRight( 5, 'c' );
 
 			assert.equal( s.toString(), '01234ABCcba56789' );
 			assert.equal( s.slice(0, 5) , '01234ABC' );
@@ -61,12 +61,12 @@ describe( 'MagicString', () => {
 			s.appendRight( 5, '}' );
 			assert.equal( s.toString(), '01234{<ABCcba>}56789' );
 
-			s.appendLeft(5, '(');   // appendLeft is a synonym for insertLeft
-			s.appendLeft(5, '[');   // appendLeft is a synonym for insertLeft
+			s.appendLeft(5, '(');
+			s.appendLeft(5, '[');
 			assert.equal( s.toString(), '01234{<ABC([cba>}56789' );
 
-			s.prependRight(5, ')'); // prependRight is a synonym for insertRight
-			s.prependRight(5, ']'); // prependRight is a synonym for insertRight
+			s.prependRight(5, ')');
+			s.prependRight(5, ']');
 			assert.equal( s.toString(), '01234{<ABC([])cba>}56789' );
 
 			assert.equal( s.slice(0, 5), '01234{<ABC([' );
@@ -141,6 +141,19 @@ describe( 'MagicString', () => {
 			assert.deepEqual( source.indentExclusionRanges, clone.indentExclusionRanges );
 		});
 
+		it( 'should clone complex indentExclusionRanges', () => {
+			const array = [ [ 3, 6 ], [ 7, 9 ] ];
+			const source = new MagicString( 'abcdefghijkl', {
+				filename: 'foo.js',
+				indentExclusionRanges: array
+			});
+
+			const clone = source.clone();
+
+			assert.notStrictEqual( source.indentExclusionRanges, clone.indentExclusionRanges );
+			assert.deepEqual( source.indentExclusionRanges, clone.indentExclusionRanges );
+		});
+
 		it( 'should clone sourcemapLocations', () => {
 			const source = new MagicString( 'abcdefghijkl', {
 				filename: 'foo.js'
@@ -266,12 +279,12 @@ describe( 'MagicString', () => {
 			assert.equal( loc.column, 9 );
 		});
 
-		it( 'should yield consistent results between insertLeft and insertRight', () => {
+		it( 'should yield consistent results between appendLeft and prependRight', () => {
 			const s1 = new MagicString( 'abcdefghijkl' );
-			s1.insertLeft( 6, 'X' );
+			s1.appendLeft( 6, 'X' );
 
 			const s2 = new MagicString( 'abcdefghijkl' );
-			s2.insertRight( 6, 'X' );
+			s2.prependRight( 6, 'X' );
 
 			const m1 = s1.generateMap({ file: 'output', source: 'input', includeContent: true });
 			const m2 = s2.generateMap({ file: 'output', source: 'input', includeContent: true });
@@ -282,7 +295,7 @@ describe( 'MagicString', () => {
 		it( 'should recover original names', () => {
 			const s = new MagicString( 'function Foo () {}' );
 
-			s.overwrite( 9, 12, 'Bar', true );
+			s.overwrite( 9, 12, 'Bar', { storeName: true });
 
 			const map = s.generateMap({
 				file: 'output.js',
@@ -298,7 +311,7 @@ describe( 'MagicString', () => {
 
 		it( 'should generate one segment per replacement', () => {
 			const s = new MagicString( 'var answer = 42' );
-			s.overwrite( 4, 10, 'number', true );
+			s.overwrite( 4, 10, 'number', { storeName: true });
 
 			const map = s.generateMap({
 				file: 'output.js',
@@ -508,7 +521,7 @@ describe( 'MagicString', () => {
 			assert.throws( () => s.insert( 6, 'X' ), /deprecated/ );
 		});
 
-		// TODO move this into insertRight and insertLeft tests
+		// TODO move this into prependRight and appendLeft tests
 
 		// it( 'should insert characters in the correct location', () => {
 		// 	const s = new MagicString( 'abcdefghijkl' );
@@ -584,9 +597,9 @@ describe( 'MagicString', () => {
 
 		it( 'ignores redundant move', () => {
 			const s = new MagicString( 'abcdefghijkl' );
-			s.insertRight( 9, 'X' );
+			s.prependRight( 9, 'X' );
 			s.move( 9, 12, 6 );
-			s.insertLeft( 12, 'Y' );
+			s.appendLeft( 12, 'Y' );
 			s.move( 6, 9, 12 ); // this is redundant – [6,9] is already after [9,12]
 
 			assert.equal( s.toString(), 'abcdefXjklYghi' );
@@ -655,7 +668,7 @@ describe( 'MagicString', () => {
 		// it( 'move follows inserts', () => {
 		// 	const s = new MagicString( 'abcdefghijkl' );
 		//
-		// 	s.insertLeft( 3, 'X' ).move( 6, 9, 3 );
+		// 	s.appendLeft( 3, 'X' ).move( 6, 9, 3 );
 		// 	assert.equal( s.toString(), 'abcXghidefjkl' );
 		// });
 		//
@@ -676,7 +689,7 @@ describe( 'MagicString', () => {
 		it( 'moves content inserted at end of range', () => {
 			const s = new MagicString( 'abcdefghijkl' );
 
-			s.insertLeft( 6, 'X' ).move( 3, 6, 9 );
+			s.appendLeft( 6, 'X' ).move( 3, 6, 9 );
 			assert.equal( s.toString(), 'abcghidefXjkl' );
 		});
 
@@ -727,7 +740,7 @@ describe( 'MagicString', () => {
 			const s = new MagicString( 'abcdefghijkl' );
 
 			s.remove( 0, 6 );
-			s.insertLeft( 6, 'DEF' );
+			s.appendLeft( 6, 'DEF' );
 			s.overwrite( 6, 9, 'GHI' );
 			assert.equal( s.toString(), 'DEFGHIjkl' );
 		});
@@ -735,7 +748,7 @@ describe( 'MagicString', () => {
 		it( 'replaces zero-length inserts inside overwrite', () => {
 			const s = new MagicString( 'abcdefghijkl' );
 
-			s.insertLeft( 6, 'XXX' );
+			s.appendLeft( 6, 'XXX' );
 			s.overwrite( 3, 9, 'DEFGHI' );
 			assert.equal( s.toString(), 'abcDEFGHIjkl' );
 		});
@@ -759,7 +772,7 @@ describe( 'MagicString', () => {
 
 		it( 'should disallow overwriting zero-length ranges', () => {
 			const s = new MagicString( 'x' );
-			assert.throws( () => s.overwrite( 0, 0, 'anything' ), /Cannot overwrite a zero-length range – use insertLeft or insertRight instead/ );
+			assert.throws( () => s.overwrite( 0, 0, 'anything' ), /Cannot overwrite a zero-length range – use appendLeft or prependRight instead/ );
 		});
 
 		it( 'should throw when given non-string content', () => {
@@ -767,16 +780,43 @@ describe( 'MagicString', () => {
 			assert.throws( () => s.overwrite( 0, 1, [] ), TypeError );
 		});
 
-		it ( 'replaces interior inserts', () => {
+		it( 'replaces interior inserts', () => {
 			const s = new MagicString( 'abcdefghijkl' );
 
-			s.insertLeft( 1, '&' );
-			s.insertRight( 1, '^' );
-			s.insertLeft( 3, '!' );
-			s.insertRight( 3, '?' );
+			s.appendLeft( 1, '&' );
+			s.prependRight( 1, '^' );
+			s.appendLeft( 3, '!' );
+			s.prependRight( 3, '?' );
 			s.overwrite( 1, 3, '...' );
 			assert.equal( s.toString(), 'a&...?defghijkl' );
 		});
+
+		it( 'preserves interior inserts with `contentOnly: true`', () => {
+			const s = new MagicString( 'abcdefghijkl' );
+
+			s.appendLeft( 1, '&' );
+			s.prependRight( 1, '^' );
+			s.appendLeft( 3, '!' );
+			s.prependRight( 3, '?' );
+			s.overwrite( 1, 3, '...', { contentOnly: true });
+			assert.equal( s.toString(), 'a&^...!?defghijkl' );
+		});
+
+		it( 'disallows overwriting across moved content', () => {
+			const s = new MagicString( 'abcdefghijkl' );
+
+			s.move( 6, 9, 3 );
+			assert.throws( () => s.overwrite( 5, 7, 'XX' ), /Cannot overwrite across a split point/ );
+		});
+
+		it ( 'allows later insertions at the end', () => {
+			const s = new MagicString( 'abcdefg' );
+
+			s.appendLeft(4, '(');
+			s.overwrite( 2, 7, '' );
+			s.appendLeft(7, 'h');
+			assert.equal( s.toString(), 'abh' );
+		});
 	});
 
 	describe( 'prepend', () => {
@@ -874,8 +914,8 @@ describe( 'MagicString', () => {
 		it( 'should not remove content inserted after the end of removed range', () => {
 			const s = new MagicString( 'ab.c;' );
 
-			s.insertRight( 0, '(' );
-			s.insertRight( 4, ')' );
+			s.prependRight( 0, '(' );
+			s.prependRight( 4, ')' );
 			s.remove( 2, 4 );
 			assert.equal( s.toString(), '(ab);' );
 		});
@@ -883,10 +923,10 @@ describe( 'MagicString', () => {
 		it( 'should remove interior inserts', () => {
 			const s = new MagicString( 'abc;' );
 
-			s.insertLeft( 1, '[' );
-			s.insertRight( 1, '(' );
-			s.insertLeft( 2, ')' );
-			s.insertRight( 2, ']' );
+			s.appendLeft( 1, '[' );
+			s.prependRight( 1, '(' );
+			s.appendLeft( 2, ')' );
+			s.prependRight( 2, ']' );
 			s.remove( 1, 2 );
 			assert.equal( s.toString(), 'a[]c;' );
 		});
@@ -903,6 +943,15 @@ describe( 'MagicString', () => {
 			const s = new MagicString( 'abcdefghijkl' );
 			assert.strictEqual( s.remove( 3, 4 ), s );
 		});
+
+		it( 'removes across moved content', () => {
+			const s = new MagicString( 'abcdefghijkl' );
+
+			s.move( 6, 9, 3 );
+			s.remove( 5, 7 );
+
+			assert.equal( s.toString(), 'abchidejkl' );
+		});
 	});
 
 	describe( 'slice', () => {
@@ -933,8 +982,8 @@ describe( 'MagicString', () => {
 		it( 'includes inserted characters, respecting insertion direction', () => {
 			const s = new MagicString( 'abefij' );
 
-			s.insertRight( 2, 'cd' );
-			s.insertLeft( 4, 'gh' );
+			s.prependRight( 2, 'cd' );
+			s.appendLeft( 4, 'gh' );
 
 			assert.equal( s.slice(), 'abcdefghij' );
 			assert.equal( s.slice( 1, 5 ), 'bcdefghi' );
